import { App, Secret, SecretParameter, Stack, StackProps } from '@aws-cdk/cdk';
import { RestApi } from '@aws-cdk/aws-apigateway';
import * as CM from '@aws-cdk/aws-certificatemanager';
import * as CF from '@aws-cdk/aws-cloudfront';
import * as CB from '@aws-cdk/aws-codebuild';
import { GitHubSourceAction, Pipeline } from '@aws-cdk/aws-codepipeline';
import * as Events from '@aws-cdk/aws-events';
import * as IAM from '@aws-cdk/aws-iam';
import { Function as Lambda } from '@aws-cdk/aws-lambda';
import { LogGroup } from '@aws-cdk/aws-logs';
import * as S3 from '@aws-cdk/aws-s3';
export declare class SeagullStack extends Stack {
    constructor(app: App, projectName: string, stackProps?: StackProps);
    addLogGroup(logGroupName: string): LogGroup;
    addS3(bucketName: string, role?: IAM.IPrincipal): S3.Bucket;
    addUniversalLambda(lambdaName: string, folder: string, role: IAM.Role): Lambda;
    addUniversalApiGateway(apiGWName: string, lambda: Lambda, stageName: string): RestApi;
    addIAMRole(roleName: string, principleName: string, actions: string[]): IAM.Role;
    addCloudfront(cfdName: string, props: CloudfrontProps): CF.CloudFrontWebDistribution;
    addPipeline(pipelineName: string): Pipeline;
    addSourceStage(name: string, config: SourceStageConfig): GitHubSourceAction;
    addBuildStage(name: string, config: BuildStageConfig): CB.PipelineBuildAction;
    addSecretParam(name: string, ssmParameter: string): SecretParameter;
    addNewCert(name: string, domains: string[]): CM.Certificate;
    importS3(bucketName: string, role?: IAM.IPrincipal): S3.BucketRef;
    addEventRule(rule: Rule, target: Events.IEventRuleTarget): Events.EventRule;
}
export interface Rule {
    path: string;
    cron: string;
}
interface StageConfig {
    atIndex: number;
    pipeline: Pipeline;
}
interface BuildStageConfig extends StageConfig {
    build: {
        commands: string[];
        finally: string[];
    };
    install: {
        commands: string[];
        finally: string[];
    };
    postBuild: {
        commands: string[];
        finally: string[];
    };
    role: IAM.Role;
    env: {
        variables: {
            [key: string]: string;
        };
    };
}
interface SourceStageConfig extends StageConfig {
    branch: string;
    owner: string;
    repo: string;
    oauthToken: Secret;
}
interface CloudfrontProps {
    apiGateway: RestApi;
    aliasConfig?: CF.AliasConfiguration;
}
export {};
