"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const cdk_1 = require("@aws-cdk/cdk");
const testing_1 = require("@seagull/testing");
require("chai/register-should");
const mocha_typescript_1 = require("mocha-typescript");
const src_1 = require("../src");
const template_searching_1 = require("./test-helper/template_searching");
let Test = class Test extends testing_1.BasicTest {
    before() {
        return __awaiter(this, void 0, void 0, function* () {
            yield testing_1.BasicTest.prototype.before.bind(this)();
        });
    }
    after() {
        return __awaiter(this, void 0, void 0, function* () {
            yield testing_1.BasicTest.prototype.after.bind(this)();
        });
    }
    'can create a stack with props'() {
        return __awaiter(this, void 0, void 0, function* () {
            const stackName = 'test-stack';
            const app = new cdk_1.App();
            const stackProps = { env: { account: '123456', region: 'my-region-1' } };
            new src_1.SeagullStack(app, stackName, stackProps);
            const synth = app.synthesizeStack(stackName);
            synth.name.should.be.equal(stackName);
            synth.environment.account.should.be.equals('123456');
            synth.environment.region.should.be.equals('my-region-1');
        });
    }
    'can add a log group to a stack'() {
        return __awaiter(this, void 0, void 0, function* () {
            const stackName = 'test-stack';
            const logGroupName = 'my-log-group';
            const app = new cdk_1.App();
            const stack = new src_1.SeagullStack(app, stackName);
            stack.addLogGroup(logGroupName);
            const synth = app.synthesizeStack(stackName);
            const resources = Object.keys(synth.template.Resources);
            const metadata = Object.keys(synth.metadata);
            const stackNameNoDash = stackName.replace(/-/g, '');
            const logGroupNameNoDash = logGroupName.replace(/-/g, '');
            const logInTemp = template_searching_1.isInList(resources, logGroupNameNoDash, stackNameNoDash);
            const logInMeta = template_searching_1.isInList(metadata, logGroupName, stackName);
            logInTemp.should.be.equals(true);
            logInMeta.should.be.equals(true);
        });
    }
    'can add a role to stack'() {
        return __awaiter(this, void 0, void 0, function* () {
            const stackName = 'test-stack';
            const roleName = 'test-role';
            const app = new cdk_1.App();
            const stack = new src_1.SeagullStack(app, stackName);
            stack.addIAMRole(roleName, 'lambda.amazonaws.com', ['action1', 'action2']);
            const synth = app.synthesizeStack(stackName);
            const resources = Object.keys(synth.template.Resources);
            const metadata = Object.keys(synth.metadata);
            const stackNoDash = stackName.replace(/-/g, '');
            const roleNoDash = roleName.replace(/-/g, '');
            const defPolicy = 'DefaultPolicy';
            const roleInTemp = template_searching_1.isInList(resources, roleNoDash, stackNoDash, defPolicy);
            const roleInMeta = template_searching_1.isInList(metadata, roleName, stackName, defPolicy);
            roleInTemp.should.be.equals(true);
            roleInMeta.should.be.equals(true);
        });
    }
    'can add a s3 bucket to a stack'() {
        return __awaiter(this, void 0, void 0, function* () {
            const stackName = 'test-stack';
            const s3Name = 'my-s3';
            const app = new cdk_1.App();
            const stack = new src_1.SeagullStack(app, stackName);
            stack.addS3(s3Name);
            const synth = app.synthesizeStack(stackName);
            const resources = Object.keys(synth.template.Resources);
            const metadata = Object.keys(synth.metadata);
            const stackNameNoDash = stackName.replace(/-/g, '');
            const s3NameNoDash = s3Name.replace(/-/g, '');
            const s3InTemp = template_searching_1.isInList(resources, s3NameNoDash, stackNameNoDash);
            const s3InMeta = template_searching_1.isInList(metadata, s3Name, stackName);
            s3InTemp.should.be.equals(true);
            s3InMeta.should.be.equals(true);
        });
    }
    'can add a s3 bucket and a role to a stack'() {
        return __awaiter(this, void 0, void 0, function* () {
            const stackName = 'test-stack';
            const s3Name = 'my-s3';
            const roleName = 'test-role';
            const app = new cdk_1.App();
            const stack = new src_1.SeagullStack(app, stackName);
            const role = stack.addIAMRole(roleName, 'lambda.amazonaws.com', []);
            stack.addS3(s3Name, role);
            const synth = app.synthesizeStack(stackName);
            const resources = Object.keys(synth.template.Resources);
            const metadata = Object.keys(synth.metadata);
            const stackNoDash = stackName.replace(/-/g, '');
            const s3NameNoDash = s3Name.replace(/-/g, '');
            const roleNoDash = roleName.replace(/-/g, '');
            const defPolicy = 'DefaultPolicy';
            const s3InTemp = template_searching_1.isInList(resources, s3NameNoDash, stackNoDash);
            const s3InMeta = template_searching_1.isInList(metadata, s3Name, stackName);
            const roleInTemp = template_searching_1.isInList(resources, roleNoDash, stackNoDash, defPolicy);
            const roleInMeta = template_searching_1.isInList(metadata, roleName, stackName, defPolicy);
            s3InTemp.should.be.equals(true);
            s3InMeta.should.be.equals(true);
            roleInTemp.should.be.equals(true);
            roleInMeta.should.be.equals(true);
        });
    }
    'can add a lambda'() {
        return __awaiter(this, void 0, void 0, function* () {
            const stackName = 'test-stack';
            const lambdaName = 'my-lambda';
            const roleName = 'test-role';
            const app = new cdk_1.App();
            const stack = new src_1.SeagullStack(app, stackName);
            const role = stack.addIAMRole(roleName, 'lambda.amazonaws.com', []);
            stack.addUniversalLambda(lambdaName, './test_data', role);
            const synth = app.synthesizeStack(stackName);
            const resources = Object.keys(synth.template.Resources);
            const metadata = Object.keys(synth.metadata);
            const stackNoDash = stackName.replace(/-/g, '');
            const lambdaNameNoDash = lambdaName.replace(/-/g, '');
            const roleNoDash = roleName.replace(/-/g, '');
            const defPolicy = 'DefaultPolicy';
            const lambdaInTemp = template_searching_1.isInList(resources, lambdaNameNoDash, stackNoDash);
            const lambdaInMeta = template_searching_1.isInList(metadata, lambdaName, stackName);
            const roleInTemp = template_searching_1.isInList(resources, roleNoDash, stackNoDash, defPolicy);
            const roleInMeta = template_searching_1.isInList(metadata, roleName, stackName, defPolicy);
            lambdaInTemp.should.be.equals(true);
            lambdaInMeta.should.be.equals(true);
            roleInTemp.should.be.equals(true);
            roleInMeta.should.be.equals(true);
        });
    }
    'can add a api gateway'() {
        return __awaiter(this, void 0, void 0, function* () {
            const stackName = 'test-stack';
            const lambdaName = 'my-lambda';
            const apiGatewayName = 'my-api-gateway';
            const roleName = 'test-role';
            const app = new cdk_1.App();
            const stack = new src_1.SeagullStack(app, stackName);
            const role = stack.addIAMRole(roleName, 'lambda.amazonaws.com', []);
            const lambda = stack.addUniversalLambda(lambdaName, './test_data', role);
            stack.addUniversalApiGateway(apiGatewayName, lambda, 'prod');
            const synth = app.synthesizeStack(stackName);
            const resources = Object.keys(synth.template.Resources);
            const metadata = Object.keys(synth.metadata);
            const stackNoDash = stackName.replace(/-/g, '');
            const gatewayNameNoDash = apiGatewayName.replace(/-/g, '');
            const gatewayInTemp = template_searching_1.isInList(resources, gatewayNameNoDash, stackNoDash);
            const gatewayInMeta = template_searching_1.isInList(metadata, apiGatewayName, stackName);
            gatewayInTemp.should.be.equals(true);
            gatewayInMeta.should.be.equals(true);
        });
    }
    'can add a cloudfront'() {
        return __awaiter(this, void 0, void 0, function* () {
            const stackName = 'test-stack';
            const lambdaName = 'my-lambda';
            const apiGatewayName = 'my-api-gateway';
            const cloudfrontName = 'my-cloudfront';
            const roleName = 'test-role';
            const app = new cdk_1.App();
            const stack = new src_1.SeagullStack(app, stackName);
            const role = stack.addIAMRole(roleName, 'lambda.amazonaws.com', []);
            const lambda = stack.addUniversalLambda(lambdaName, './test_data', role);
            const apiGW = stack.addUniversalApiGateway(apiGatewayName, lambda, 'prod');
            stack.addCloudfront(cloudfrontName, { apiGateway: apiGW });
            const synth = app.synthesizeStack(stackName);
            const resources = Object.keys(synth.template.Resources);
            const metadata = Object.keys(synth.metadata);
            const stackNoDash = stackName.replace(/-/g, '');
            const cloudfrontNameNoDash = cloudfrontName.replace(/-/g, '');
            const cloudfrInTemp = template_searching_1.isInList(resources, cloudfrontNameNoDash, stackNoDash);
            const cloudfrMeta = template_searching_1.isInList(metadata, cloudfrontName, stackName);
            cloudfrInTemp.should.be.equals(true);
            cloudfrMeta.should.be.equals(true);
        });
    }
    'can add a pipeline'() {
        return __awaiter(this, void 0, void 0, function* () {
            const stackName = 'test-stack';
            const pipelineName = 'test-pipeline';
            const sourceName = 'test-source';
            const buildName = 'test-build';
            const roleName = 'test-role';
            const app = new cdk_1.App();
            const stack = new src_1.SeagullStack(app, stackName);
            const role = stack.addIAMRole(roleName, 'codebuild.amazonaws.com', []);
            const pipeline = stack.addPipeline(pipelineName);
            const sourceConfig = {
                atIndex: 0,
                branch: 'master',
                oauthToken: new cdk_1.Secret('12345678'),
                owner: 'me',
                pipeline,
                repo: 'test-project',
            };
            stack.addSourceStage(sourceName, sourceConfig);
            const buildConfig = {
                atIndex: 1,
                build: { commands: ['echo "start build"'], finally: [] },
                env: { variables: {} },
                install: { commands: ['npm i'], finally: [] },
                pipeline,
                postBuild: { commands: ['npm run test'], finally: [] },
                role,
            };
            stack.addBuildStage(buildName, buildConfig);
            const synth = app.synthesizeStack(stackName);
            const resources = Object.keys(synth.template.Resources);
            const metadata = Object.keys(synth.metadata);
            const stackNoDash = stackName.replace(/-/g, '');
            const pipelineNameNoDash = pipelineName.replace(/-/g, '');
            const sourceNameNoDash = sourceName.replace(/-/g, '');
            const buildNameNoDash = buildName.replace(/-/g, '');
            const sourceInTemp = template_searching_1.isInList(resources, sourceNameNoDash, stackNoDash);
            const sourceMeta = template_searching_1.isInList(metadata, sourceName, stackName);
            const buildInTemp = template_searching_1.isInList(resources, buildNameNoDash, stackNoDash);
            const buildMeta = template_searching_1.isInList(metadata, buildName, stackName);
            const pipelineInTemp = template_searching_1.isInList(resources, pipelineNameNoDash, stackNoDash);
            const pipelineMeta = template_searching_1.isInList(metadata, pipelineName, stackName);
            sourceInTemp.should.be.equals(true);
            sourceMeta.should.be.equals(true);
            buildInTemp.should.be.equals(true);
            buildMeta.should.be.equals(true);
            pipelineInTemp.should.be.equals(true);
            pipelineMeta.should.be.equals(true);
        });
    }
    'can add a secret paramter'() {
        return __awaiter(this, void 0, void 0, function* () {
            const stackName = 'test-stack';
            const secretName = 'test-secret';
            const app = new cdk_1.App();
            const stack = new src_1.SeagullStack(app, stackName);
            stack.addSecretParam(secretName, 'my-secret-key');
            const secretNameNoDash = secretName.replace(/-/g, '');
            const synth = app.synthesizeStack(stackName);
            const parameters = Object.keys(synth.template.Parameters);
            const metadata = Object.keys(synth.metadata);
            const secretInParameters = template_searching_1.isInList(parameters, secretNameNoDash);
            const secretMeta = template_searching_1.isInList(metadata, secretName, stackName);
            secretInParameters.should.be.equals(true);
            secretMeta.should.be.equals(true);
        });
    }
};
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], Test.prototype, "can create a stack with props", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], Test.prototype, "can add a log group to a stack", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], Test.prototype, "can add a role to stack", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], Test.prototype, "can add a s3 bucket to a stack", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], Test.prototype, "can add a s3 bucket and a role to a stack", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], Test.prototype, "can add a lambda", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], Test.prototype, "can add a api gateway", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], Test.prototype, "can add a cloudfront", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], Test.prototype, "can add a pipeline", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], Test.prototype, "can add a secret paramter", null);
Test = __decorate([
    mocha_typescript_1.suite('SeagullStack')
], Test);
exports.Test = Test;
//# sourceMappingURL=seagull_stack.js.map