"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const aws_iam_1 = require("@aws-cdk/aws-iam");
const commands_fs_1 = require("@seagull/commands-fs");
const testing_1 = require("@seagull/testing");
const chai_1 = require("chai");
const promisedChai = require("chai-as-promised");
require("chai/register-should");
const lodash_1 = require("lodash");
const mocha_typescript_1 = require("mocha-typescript");
const src_1 = require("../../src");
const Handlers = require("../../src/aws_sdk_handler");
const template_searching_1 = require("../test-helper/template_searching");
chai_1.use(promisedChai);
const s3Name = 'another-s3';
const customizationCode = `import { SeagullApp } from '../src'
export default function(app: SeagullApp) {
  app.stack.addS3('another-s3', app.role)
}`;
let Test = class Test extends testing_1.BasicTest {
    constructor() {
        super(...arguments);
        this.appPath = `${process.cwd()}/test_data`;
    }
    before() {
        return __awaiter(this, void 0, void 0, function* () {
            yield testing_1.BasicTest.prototype.before.bind(this)();
            process.env.AWS_REGION = 'eu-central-1';
            const assetFolder = `${this.appPath}/dist/assets`;
            const backendFolder = `${assetFolder}/backend`;
            const createBackendFolder = new commands_fs_1.FS.CreateFolder(backendFolder);
            yield createBackendFolder.execute();
            yield new commands_fs_1.FS.WriteFile(`${backendFolder}/server.js`, '').execute();
            yield new commands_fs_1.FS.WriteFile(`${backendFolder}/lambda.js`, '').execute();
            yield new commands_fs_1.FS.WriteFile(`${this.appPath}/dist/cron.json`, JSON.stringify([])).execute();
        });
    }
    after() {
        return __awaiter(this, void 0, void 0, function* () {
            yield testing_1.BasicTest.prototype.after.bind(this)();
        });
    }
    'can create a project'() {
        return __awaiter(this, void 0, void 0, function* () {
            const props = getTestProps(this.appPath);
            const project = yield new src_1.SeagullProject(props).createSeagullApp();
            const synthStack = project.synthesizeStack('helloworld');
            Object.keys(synthStack.template.Resources).length.should.be.above(1);
        });
    }
    'assigns a default role to role property of SeagullApp'() {
        return __awaiter(this, void 0, void 0, function* () {
            const props = getTestProps(this.appPath);
            const app = yield new src_1.SeagullProject(props).createSeagullApp();
            chai_1.expect(app.role).to.be.instanceOf(aws_iam_1.Role);
        });
    }
    'can add policies after creating SeagullApp'() {
        return __awaiter(this, void 0, void 0, function* () {
            const props = getTestProps(this.appPath);
            const app = yield new src_1.SeagullProject(props).createSeagullApp();
            const stmt = new aws_iam_1.PolicyStatement().addAllResources().addAction('action3');
            app.role.addToPolicy(stmt);
            const synth = app.synthesizeStack('helloworld');
            const newPolicyCriterion = resourceHasNewAction('action3');
            const hasNewPolicy = !!lodash_1.find(synth.template.Resources, newPolicyCriterion);
            hasNewPolicy.should.be.equal(true);
        });
    }
    'can create a project and customize stack'() {
        return __awaiter(this, void 0, void 0, function* () {
            const props = getTestProps(this.appPath);
            yield writeCustomInfraFile(this.appPath);
            const project = new src_1.SeagullProject(props);
            const app = yield project.createSeagullApp();
            const stackName = 'helloworld';
            yield project.customizeStack(app);
            const synthStack = app.synthesizeStack(stackName);
            const resources = Object.keys(synthStack.template.Resources);
            const metadata = Object.keys(synthStack.metadata);
            const stackNameNoDash = stackName.replace(/-/g, '');
            const s3NameNoDash = s3Name.replace(/-/g, '');
            const s3InTemp = template_searching_1.isInList(resources, s3NameNoDash, stackNameNoDash);
            const s3InMeta = template_searching_1.isInList(metadata, s3Name, stackName);
            s3InTemp.should.be.equal(true);
            s3InMeta.should.be.equal(true);
            yield deleteCustomInfra(this.appPath);
        });
    }
    'customize stack throws when infrastructure-aws.ts is corrupted'() {
        return __awaiter(this, void 0, void 0, function* () {
            const newCWD = `${this.appPath}/corrupted-infrastructure`;
            const assetFolder = `${newCWD}/dist/assets`;
            const backendFolder = `${assetFolder}/backend`;
            const createBackendFolder = new commands_fs_1.FS.CreateFolder(backendFolder);
            yield createBackendFolder.execute();
            yield new commands_fs_1.FS.WriteFile(`${backendFolder}/server.js`, '').execute();
            yield new commands_fs_1.FS.WriteFile(`${backendFolder}/lambda.js`, '').execute();
            yield new commands_fs_1.FS.WriteFile(`${newCWD}/dist/cron.json`, '[]').execute();
            yield writeCustomInfraFile(newCWD, 'export default 123');
            const props = getTestProps(newCWD);
            const project = new src_1.SeagullProject(props);
            const app = yield project.createSeagullApp();
            const customize = () => project.customizeStack(app);
            yield chai_1.expect(customize()).to.be.rejectedWith(Error);
            yield deleteCustomInfra(`${this.appPath}/corrupted-infrastructure`);
        });
    }
    'customizeStack should do nothing but return false without an infrastructure-aws.ts-file'() {
        return __awaiter(this, void 0, void 0, function* () {
            const props = getTestProps(this.appPath);
            const project = new src_1.SeagullProject(props);
            const app = yield project.createSeagullApp();
            const appBeforeCustomization = lodash_1.cloneDeep(app);
            const returnValue = yield project.customizeStack(app);
            chai_1.expect(app).to.deep.equal(appBeforeCustomization);
            chai_1.expect(returnValue).to.be.equal(false);
        });
    }
    'can deploy a project without customized stack'() {
        return __awaiter(this, void 0, void 0, function* () {
            const props = getTestProps(this.appPath);
            const project = new src_1.SeagullProject(props);
            let hasBeenCalled = false;
            const deployStack = () => (hasBeenCalled = true);
            project.createSeagullApp = () => Promise.resolve({ deployStack });
            yield project.deployProject();
            chai_1.expect(hasBeenCalled).to.be.equal(true);
        });
    }
    'can deploy a project with customized stack'() {
        return __awaiter(this, void 0, void 0, function* () {
            yield writeCustomInfraFile(this.appPath);
            const props = getTestProps(this.appPath);
            const project = new src_1.SeagullProject(props);
            let hasBeenCalled = false;
            const deployStack = () => __awaiter(this, void 0, void 0, function* () {
                hasBeenCalled = true;
            });
            const createSeagullApp = project.createSeagullApp;
            let app;
            project.createSeagullApp = () => __awaiter(this, void 0, void 0, function* () {
                app = yield createSeagullApp.bind(project)();
                app.deployStack = deployStack;
                return app;
            });
            const stackName = 'helloworld';
            yield project.deployProject();
            const synthStack = app.synthesizeStack(stackName);
            const resources = Object.keys(synthStack.template.Resources);
            const metadata = Object.keys(synthStack.metadata);
            const stackNameNoDash = stackName.replace(/-/g, '');
            const s3NameNoDash = s3Name.replace(/-/g, '');
            const s3InTemp = template_searching_1.isInList(resources, s3NameNoDash, stackNameNoDash);
            const s3InMeta = template_searching_1.isInList(metadata, s3Name, stackName);
            s3InTemp.should.be.equal(true);
            s3InMeta.should.be.equal(true);
            hasBeenCalled.should.be.equal(true);
            yield deleteCustomInfra(this.appPath);
        });
    }
    'can diff a project without customized stack'() {
        return __awaiter(this, void 0, void 0, function* () {
            const props = getTestProps(this.appPath);
            const project = new src_1.SeagullProject(props);
            let hasBeenCalled = false;
            const diffStack = () => (hasBeenCalled = true);
            project.createSeagullApp = () => Promise.resolve({ diffStack });
            yield project.diffProject();
            chai_1.expect(hasBeenCalled).to.be.equal(true);
        });
    }
    'can diff a project with customized stack'() {
        return __awaiter(this, void 0, void 0, function* () {
            yield writeCustomInfraFile(this.appPath);
            const props = getTestProps(this.appPath);
            const project = new src_1.SeagullProject(props);
            let hasBeenCalled = false;
            const diffStack = () => __awaiter(this, void 0, void 0, function* () {
                hasBeenCalled = true;
            });
            const createSeagullApp = project.createSeagullApp;
            let app;
            project.createSeagullApp = () => __awaiter(this, void 0, void 0, function* () {
                app = yield createSeagullApp.bind(project)();
                app.diffStack = diffStack;
                return app;
            });
            const stackName = 'helloworld';
            yield project.diffProject();
            const synthStack = app.synthesizeStack(stackName);
            const resources = Object.keys(synthStack.template.Resources);
            const metadata = Object.keys(synthStack.metadata);
            const stackNameNoDash = stackName.replace(/-/g, '');
            const s3NameNoDash = s3Name.replace(/-/g, '');
            const s3InTemp = template_searching_1.isInList(resources, s3NameNoDash, stackNameNoDash);
            const s3InMeta = template_searching_1.isInList(metadata, s3Name, stackName);
            s3InTemp.should.be.equal(true);
            s3InMeta.should.be.equal(true);
            hasBeenCalled.should.be.equal(true);
            yield deleteCustomInfra(this.appPath);
        });
    }
};
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], Test.prototype, "can create a project", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], Test.prototype, "assigns a default role to role property of SeagullApp", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], Test.prototype, "can add policies after creating SeagullApp", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], Test.prototype, "can create a project and customize stack", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], Test.prototype, "customize stack throws when infrastructure-aws.ts is corrupted", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], Test.prototype, "customizeStack should do nothing but return false without an infrastructure-aws.ts-file", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], Test.prototype, "can deploy a project without customized stack", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], Test.prototype, "can deploy a project with customized stack", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], Test.prototype, "can diff a project without customized stack", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], Test.prototype, "can diff a project with customized stack", null);
Test = __decorate([
    mocha_typescript_1.suite('SeagullProject')
], Test);
exports.Test = Test;
class TestACMHandler extends Handlers.ACMHandler {
    constructor(testData) {
        super();
        this.arnsWithDomains = testData || { arn1: ['www.aida.de', 'www2.aida.de'] };
    }
    listCertificates() {
        return __awaiter(this, void 0, void 0, function* () {
            return Object.keys(this.arnsWithDomains);
        });
    }
    describeCertificate(acmCertRef) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.arnsWithDomains[acmCertRef] || [];
        });
    }
}
class TestCloudfrontHandler extends Handlers.CloudfrontHandler {
    constructor(testData) {
        super();
        const defaultData = [{ Comment: 'helloworld', DomainName: 'abcdef.cf.net' }];
        this.cloudfrontList = testData || defaultData;
    }
    listDistributions() {
        return __awaiter(this, void 0, void 0, function* () {
            return this.cloudfrontList;
        });
    }
}
class TestSTSHandler extends Handlers.STSHandler {
    constructor(testAccount) {
        super();
        this.accountId = testAccount || 'test-account-id';
    }
    getAccountId() {
        return __awaiter(this, void 0, void 0, function* () {
            return this.accountId;
        });
    }
}
const getTestProps = (appPath) => ({
    appPath,
    branch: 'master',
    githubToken: 'Token123',
    handlers: {
        acmHandler: new TestACMHandler(),
        cloudfrontHandler: new TestCloudfrontHandler(),
        stsHandler: new TestSTSHandler(),
    },
    mode: 'prod',
    owner: 'me',
    profile: 'default',
    region: 'eu-central-1',
    repository: 'test-repo',
});
const deleteCustomInfra = (appPath) => __awaiter(this, void 0, void 0, function* () {
    const customInfraDel = new commands_fs_1.FS.DeleteFile(`${appPath}/infrastructure-aws.ts`);
    yield customInfraDel.execute();
    customInfraDel.mode = { environment: 'edge' };
    yield customInfraDel.execute();
});
const writeCustomInfraFile = (appPath, code = customizationCode) => __awaiter(this, void 0, void 0, function* () {
    const customizationPath = `${appPath}/infrastructure-aws.ts`;
    const customInfraGen = new commands_fs_1.FS.WriteFile(customizationPath, code);
    yield customInfraGen.execute();
    customInfraGen.mode = { environment: 'edge' };
    yield customInfraGen.execute();
});
const resPropHasNewAction = (action) => (resProp) => !!resProp.Statement &&
    !!lodash_1.find(resProp.Statement, (s) => s.Action.includes(action));
const resourceHasNewAction = (action) => (res) => !!lodash_1.find(res.Properties, resPropHasNewAction(action));
//# sourceMappingURL=seagull_project.js.map